@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: interpret.c,v 1.16 2002/12/10 00:08:32 nalin Exp $"
+#ident "$Id: interpret.c,v 1.17 2003/02/14 21:58:18 nalin Exp $"
 #include "../config.h"
 #include <sys/types.h>
 #include <assert.h>
@@ -208,6 +208,9 @@ main(int argc, char **argv)
 					g_byte_array_remove_index(array, 0);
 				}
 				for (j = 0; j < substlen; j++) {
+					if (VTE_ISO2022_HAS_ENCODED_WIDTH(ubuf[j])) {
+						ubuf[j] &= ~VTE_ISO2022_ENCODED_WIDTH_MASK;
+					}
 					if (ubuf[j] < 32) {
 						printf("`^%c'\n", ubuf[j] + 64);
 					} else
