@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: iso2022.c,v 1.20.2.1 2003/02/21 22:51:13 nalin Exp $"
+#ident "$Id: iso2022.c,v 1.22 2003/02/27 00:31:47 nalin Exp $"
 #include "../config.h"
 #include <sys/types.h>
 #include <stdio.h>
@@ -249,6 +249,39 @@ static const struct _vte_iso2022_map _vte_iso2022_map_wide_H[] = {
 #include "unitable.CNS11643"
 };
 
+#include "uniwidths"
+
+static gint
+_vte_direct_compare(gconstpointer a, gconstpointer b)
+{
+	return GPOINTER_TO_INT(a) - GPOINTER_TO_INT(b);
+}
+
+gboolean
+_vte_iso2022_is_ambiguous(gunichar c)
+{
+	int i;
+	gpointer p;
+	static GTree *ambiguous = NULL;
+	for (i = 0; i < G_N_ELEMENTS(_vte_iso2022_ambiguous_ranges); i++) {
+		if ((c >= _vte_iso2022_ambiguous_ranges[i].start) &&
+		    (c <= _vte_iso2022_ambiguous_ranges[i].end)) {
+			return TRUE;
+		}
+	}
+	if (ambiguous == NULL) {
+		ambiguous = g_tree_new(_vte_direct_compare);
+		for (i = 0;
+		     i < G_N_ELEMENTS(_vte_iso2022_ambiguous_chars);
+		     i++) {
+			p = GINT_TO_POINTER(_vte_iso2022_ambiguous_chars[i]);
+			g_tree_insert(ambiguous, p, p);
+		}
+	}
+	p = GINT_TO_POINTER(c);
+	return g_tree_lookup(ambiguous, p) != NULL;
+}
+
 struct _vte_iso2022 *
 _vte_iso2022_new(void)
 {
@@ -290,12 +323,6 @@ _vte_iso2022_free(struct _vte_iso2022 *p)
 	g_free(p);
 }
 
-static gint
-_vte_direct_compare(gconstpointer a, gconstpointer b)
-{
-	return GPOINTER_TO_INT(a) - GPOINTER_TO_INT(b);
-}
-
 static GTree *
 _vte_iso2022_map_init(const struct _vte_iso2022_map *map, gssize length)
 {
@@ -432,85 +459,106 @@ _vte_iso2022_map_get(gunichar mapname)
 }
 
 gssize
-_vte_iso2022_get_width(gunichar c)
+_vte_iso2022_get_encoded_width(gunichar c)
 {
 	gssize width;
-	width = (c & VTE_ISO2022_WIDTH_MASK) >> VTE_ISO2022_WIDTH_BIT_OFFSET;
+	width = (c & VTE_ISO2022_ENCODED_WIDTH_MASK) >> VTE_ISO2022_ENCODED_WIDTH_BIT_OFFSET;
 	return CLAMP(width, 0, 2);
 }
 
 static gunichar
-_vte_iso2022_set_width(gunichar c, gssize width)
+_vte_iso2022_set_encoded_width(gunichar c, gssize width)
 {
 	width = CLAMP(width, 0, 2);
-	c &= ~(VTE_ISO2022_WIDTH_MASK);
-	c |= (width << VTE_ISO2022_WIDTH_BIT_OFFSET);
+	c &= ~(VTE_ISO2022_ENCODED_WIDTH_MASK);
+	c |= (width << VTE_ISO2022_ENCODED_WIDTH_BIT_OFFSET);
 	return c;
 }
 
+static int
+_vte_iso2022_ambiguous_width(void)
+{
+	const char *lang = NULL;
+	int ret = 1;
+	if ((lang == NULL) && (g_getenv("LC_ALL") != NULL)) {
+		lang = g_getenv("LC_ALL");
+	}
+	if ((lang == NULL) && (g_getenv("LC_CTYPE") != NULL)) {
+		lang = g_getenv("LC_CTYPE");
+	}
+	if ((lang == NULL) && (g_getenv("LANG") != NULL)) {
+		lang = g_getenv("LANG");
+	}
+	if (lang != NULL) {
+		if (g_ascii_strncasecmp(lang, "ja", 2) == 0) {
+			ret = 2;
+		} else
+		if (g_ascii_strncasecmp(lang, "ko", 2) == 0) {
+			ret = 2;
+		} else
+		if (g_ascii_strncasecmp(lang, "vi", 2) == 0) {
+			ret = 2;
+		} else
+		if (g_ascii_strncasecmp(lang, "zh", 2) == 0) {
+			ret = 2;
+		}
+	}
+#ifdef VTE_DEBUG
+	if (_vte_debug_on(VTE_DEBUG_SUBSTITUTION)) {
+		fprintf(stderr, "Ambiguous characters will have width = %d.\n",
+			ret);
+	}
+#endif
+	return ret;
+}
+
 gunichar
 _vte_iso2022_substitute_single(gunichar mapname, gunichar c)
 {
 	GTree *charmap;
 	gunichar result = c;
-	gssize width;
+	gssize width = 0;
+	static int ambiguous_width = 0;
 	charmap = _vte_iso2022_map_get(mapname);
+	/* Map the character. */
 	if (charmap) {
 		c = GPOINTER_TO_INT(g_tree_lookup(charmap, GINT_TO_POINTER(c)));
 		if (c != 0) {
 			result = c;
 		}
 	}
-	/* Calculate width for ambiguous characters. */
-	switch (result) {
-		/* To be filled in some time, I guess. */
-	default:
-		/* For nonambiguous characters, just use
-		 * the width glib has later on. */
-		width = 0;
-		break;
-	}
-	/* Override for drawing characters. */
-	if (mapname == '0') {
-		if ((result >= 0x2500) && (result <= 0x257f)) {
-			width = 1;
-		} else
-		switch (c) {
-		case 0x00a3: /* british pound */
-		case 0x00b0: /* degree */
-		case 0x00b1: /* plus/minus */
-		case 0x00b7: /* bullet */
-		case 0x03c0: /* pi */
-		case 0x2190: /* left arrow */
-		case 0x2191: /* up arrow */
-		case 0x2192: /* right arrow */
-		case 0x2193: /* down arrow */
-		case 0x2260: /* != */
-		case 0x2264: /* <= */
-		case 0x2265: /* >= */
-		case 0x23ba: /* scanline 1/9 */
-		case 0x23bb: /* scanline 3/9 */
-		case 0x23bc: /* scanline 7/9 */
-		case 0x23bd: /* scanline 9/9 */
-		case 0x2409: /* HT symbol */
-		case 0x240a: /* LF symbol */
-		case 0x240b: /* VT symbol */
-		case 0x240c: /* FF symbol */
-		case 0x240d: /* CR symbol */
-		case 0x2424: /* NL symbol */
-		case 0x2592: /* checkerboard */
-		case 0x25ae: /* solid rectangle */
-		case 0x25c6: /* diamond */
+	/* Determine if we'll code in a width. */
+	if (result != c) {
+		switch (mapname) {
+		case '0':
+			/* All ACS-substituted characters get a width
+			 * of one. */
 			width = 1;
 			break;
 		default:
 			break;
 		}
 	}
-	/* Save the width if one was set. */
-	if (width != 0) {
-		result = _vte_iso2022_set_width(result, width);
+	/* Calculate width for ambiguous characters. */
+	if ((width == 0) && _vte_iso2022_is_ambiguous(result)) {
+		if (ambiguous_width == 0) {
+			ambiguous_width = _vte_iso2022_ambiguous_width();
+		}
+		width = ambiguous_width;
+#ifdef VTE_DEBUG
+		if (_vte_debug_on(VTE_DEBUG_SUBSTITUTION)) {
+			fprintf(stderr, "Character U%lx has ambiguous width.\n",
+				(long) c);
+		}
+#endif
+	} else
+	if (g_unichar_iswide(result)) {
+		width = 2;
+	} else {
+		width = 1;
 	}
+	/* Encode the width. */
+	result = _vte_iso2022_set_encoded_width(result, width);
 	return result;
 }
 
@@ -529,6 +577,7 @@ _vte_iso2022_substitute(struct _vte_iso2022 *outside_state,
 	const gunichar *used;
 	int chars_per_code = 1;
 	gssize width;
+	static int ambiguous_width = 0;
 
 	g_return_val_if_fail(outside_state != NULL, 0);
 	g_return_val_if_fail(instring != NULL, 0);
@@ -899,56 +948,32 @@ _vte_iso2022_substitute(struct _vte_iso2022 *outside_state,
 #endif
 				}
 			}
-			/* Calculate width for ambiguous characters. */
-			switch (result) {
-				/* To be filled in some time, I guess. */
-			default:
-				/* For nonambiguous characters, just use
-				 * the width glib has later on. */
-				width = 0;
-				break;
-			}
-			/* Override for drawing characters. */
-			if (current_map == '0') {
-				if ((result >= 0x2500) && (result <= 0x257f)) {
-					width = 1;
-				} else
-				switch (result) {
-				case 0x00a3: /* british pound */
-				case 0x00b0: /* degree */
-				case 0x00b1: /* plus/minus */
-				case 0x00b7: /* bullet */
-				case 0x03c0: /* pi */
-				case 0x2190: /* left arrow */
-				case 0x2191: /* up arrow */
-				case 0x2192: /* right arrow */
-				case 0x2193: /* down arrow */
-				case 0x2260: /* != */
-				case 0x2264: /* <= */
-				case 0x2265: /* >= */
-				case 0x23ba: /* scanline 1/9 */
-				case 0x23bb: /* scanline 3/9 */
-				case 0x23bc: /* scanline 7/9 */
-				case 0x23bd: /* scanline 9/9 */
-				case 0x2409: /* HT symbol */
-				case 0x240a: /* LF symbol */
-				case 0x240b: /* VT symbol */
-				case 0x240c: /* FF symbol */
-				case 0x240d: /* CR symbol */
-				case 0x2424: /* NL symbol */
-				case 0x2592: /* checkerboard */
-				case 0x25ae: /* solid rectangle */
-				case 0x25c6: /* diamond */
+			/* Determine if we'll code in a width. */
+			width = 0;
+			if (result != accumulator) {
+				switch (current_map) {
+				case '0':
+					/* All ACS-substituted characters get a
+					 * width of one. */
 					width = 1;
 					break;
 				default:
 					break;
 				}
 			}
-			/* Save the width if one was set. */
-			if (width != 0) {
-				result = _vte_iso2022_set_width(result, width);
+			/* Calculate width for ambiguous characters. */
+			if ((width == 0) && _vte_iso2022_is_ambiguous(result)) {
+				if (ambiguous_width == 0) {
+					ambiguous_width = _vte_iso2022_ambiguous_width();
+				}
+				width = ambiguous_width;
+			} else
+			if (g_unichar_iswide(result)) {
+				width = 2;
+			} else {
+				width = 1;
 			}
+			result = _vte_iso2022_set_encoded_width(result, width);
 			/* Store. */
 			buf[j++] = result;
 			accumulator = 0;
