@@ -21,7 +21,7 @@
 #ifndef vte_iso2022_h_included
 #define vte_iso2022_h_included
 
-#ident "$Id: iso2022.h,v 1.9 2002/12/14 00:40:53 nalin Exp $"
+#ident "$Id: iso2022.h,v 1.10 2003/02/11 20:21:43 nalin Exp $"
 
 #include <glib.h>
 #include <glib-object.h>
@@ -38,10 +38,11 @@ gssize _vte_iso2022_substitute(struct _vte_iso2022 *state,
 			       gunichar *outstring,
 			       struct _vte_matcher *specials);
 
-#define VTE_ISO2022_WIDTH_BIT_OFFSET	28
-#define VTE_ISO2022_WIDTH_MASK		(3 << VTE_ISO2022_WIDTH_BIT_OFFSET)
-#define VTE_ISO2022_HAS_WIDTH(__c)	(((__c) & VTE_ISO2022_WIDTH_MASK) != 0)
-gssize _vte_iso2022_get_width(gunichar c);
+#define VTE_ISO2022_ENCODED_WIDTH_BIT_OFFSET	28
+#define VTE_ISO2022_ENCODED_WIDTH_MASK		(3 << VTE_ISO2022_ENCODED_WIDTH_BIT_OFFSET)
+#define VTE_ISO2022_HAS_ENCODED_WIDTH(__c)	(((__c) & VTE_ISO2022_ENCODED_WIDTH_MASK) != 0)
+gssize _vte_iso2022_get_encoded_width(gunichar c);
+gboolean _vte_iso2022_is_ambiguous(gunichar c);
 
 G_END_DECLS
 
